/*********************************************************************/ 
/*                                                                   */ 
/*             Config files library - (C) TGG 2015                   */ 
/*                                                                   */ 
/*********************************************************************/ 
/*                          Warszawa, 2015                           */ 
/*********************************************************************/ 
/*                                                                   */
/* File: conect_data.h                                               */
/*                                                                   */
/* Author: T.Grabowski                                               */
/*                                                                   */
/* Contents - CONNECT_DATA class                                     */
/*                                                                   */ 
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/*                                                                   */

#ifndef _CONECT_DATA_H_
#define _CONECT_DATA_H_

#include <fstream>

#include "iofun.h"
#include "ms2data_struct.h"

/// Structure, which defines connection of two wings.

/// The connection can be vertical, horizontal or parallel.
/// Details are in PANUKL manual.

class CONNECT_DATA : public IOFUN
{
	private :
	char* TypeStr[3];

	public:

	CONNECT_DATA( void );
	~CONNECT_DATA( void );

	int iTyp;       ///< Type of connection ( horizontal, vertical, parallel )
	int iActive;    ///< is connection section active ( 1- active, 2 - not active)
	int CP1;        ///< number of the 1st wing (vertical in case of horizontal connection)
	int CP2;        ///< number of the 2nd wing (horizontal in case of vertical connection)
	int Czeb1;      ///< number of wing 1 rib connected to wing 2	
	int Czeb2;      ///< number of wing 2 rib connected to wing 1
	int Cstart;     ///< number of wing 2 longeron connected to leading point of wing 1
	int Cend;       ///< number of wing 2 longeron connected to trailing point of wing 1
	int Csurf;      ///< surface of wing 2 which will be modified (0-bottom or exterior, 1-top or interior)

/// Name of connection
	std::string ConName;

/// Read connection section (within MS2 file reading)
	int Read( FILE *f, int typ, int version = 2 );
/// Write connection section (within MS2 file saving)
	int Write( std::ostream &fout );
};

#endif

