/*********************************************************************/
/*                                                                   */
/*              Config files library  - (C) TGG 2015                 */
/*                                                                   */
/*********************************************************************/
/*                          Warsaw 2008                              */
/*********************************************************************/
/*                                                                   */
/* File: memfun.h                                                    */
/*                                                                   */
/* Written by: F.A.Dul, T.Grabowski                                  */
/*                                                                   */
/* Contents: smart memory functions, operators, macros               */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/*                                                                   */

/** \file
    This file contains some smart functions, operators, macros for safe initializing end erasing the memory
*/

#ifndef _MEMFUN_H_
#define _MEMFUN_H_

#include <memory>
#include <cstdlib>
#include <cstring>
#include <iostream>

/// class with overloaded "new" operator initializing object with zeros

class MEMFUN
{
    public:

/// 
/// \brief Overloaded "new" operator initializing object with zeros
/// \param text - optional comment to be printed to stderr
/// \param ierr_print - flag that controls the message printing to stderr (1 - prints, 0 - doesn't print)
///
	   static void* operator new ( size_t st , const char *text = "(unknown)", int ierr_print = 1 );
	   
};

/// 
/// \brief Safe erasing of the object and sets the pointer to zero
/// \param ptr - reference to the object pointer
/// \param text - optional comment to be printed to stderr
///

template <class typ> void ERASE( typ **ptr , const char *text = "(No name specified)" )
{
    if( *ptr )
      {
      //std::cerr << "Delete performed ptr " << (int)*ptr << text << std::endl;
      delete *ptr;
      *ptr = NULL;
      }
    else
      std::cerr << "Warning! Pointer to : " << text << " is NULL !!! No delete performed" << std::endl;
}

/// 
/// \brief Macro for initializing array with zeros
/// \param Var - pointer to the array
/// \param Typ - type of array (e.g. double)
/// \param Len - legth of array
///

#define NEW_TAB( Var, Typ, Len ) \
{ \
    (Var) = new Typ[Len]; \
    std::cerr << " Creating object: " << #Var << " address = " << Var << " typ: " << #Typ << " size (type units) = " << Len << std::endl; \
    memset( (Var), 0, sizeof(Typ)*(Len) ); \
}

/// 
/// \brief Macro for safe deleteing the object and sets the pointer to zero
/// \param OBJ - pointer to the object
///

#ifndef DELETE_
#define DELETE_( OBJ ) \
{ \
    if( OBJ ) {delete OBJ; OBJ = NULL;} \
}
#endif
//    if( OBJ ) std::cerr << " Delete object: " << #OBJ << " address = " << OBJ << std::endl;

/// 
/// \brief Macro for safe deleteing the array and sets the pointer to zero
/// \param OBJ - pointer to the array
///

#ifndef DELETE_TAB
#define DELETE_TAB( OBJ ) \
{ \
    if( OBJ ) std::cerr << " Delete tab: " << #OBJ << " address = " << OBJ << std::endl; \
    if( OBJ ) {delete [] OBJ; OBJ = NULL;} \
}
#endif

#endif /*_MEMFUN_H_*/
