/*********************************************************************/
/*                                                                   */
/*                Panukl package - (C) TGG 2002                      */
/*             Config files library - (C) TGG 2015                   */
/*                                                                   */
/*********************************************************************/
/*                          Warszawa, 2011                           */
/*********************************************************************/
/*                                                                   */
/* File: panel.cpp                                                   */
/*                                                                   */
/* Author: T.Grabowski                                               */
/*                                                                   */
/* Contents - PANEL class                                            */
/*                                                                   */
/* Last update: 6.04.2020                                            */
/*                                                                   */
/*********************************************************************/
/*                                                                   */

#include "panel.h"

PANEL::PANEL( void )
{
	Clear();
}

PANEL::PANEL( VECTOR_3D vP1, VECTOR_3D vP2, VECTOR_3D vP3, VECTOR_3D vP4 )
{
	Clear();
	PutTo( vP1, vP2, vP3, vP4 );
}

PANEL::~PANEL( void )
{
	Clear();
}

void PANEL::Clear( void )
{
	P1 = VECTOR_3__E0();
	P2 = VECTOR_3__E0();
	P3 = VECTOR_3__E0();
	P4 = VECTOR_3__E0();
	PP[0] = &P1;
	PP[1] = &P2;
	PP[2] = &P3;
	PP[3] = &P4;
	strcpy(ElSet, "");
	Type = 0;
	Pc = VECTOR_3__E0();
	IdP1 = IdP2 = IdP3 = IdP4 = IdP5 = IdP6 = IdP7 = IdP8 = 0;
}

void PANEL::PutTo( VECTOR_3D vP1, VECTOR_3D vP2, VECTOR_3D vP3, VECTOR_3D vP4 )
{
	P1 = vP1;
	P2 = vP2;
	P3 = vP3;
	P4 = vP4;
	Center();
}

void PANEL::PutTo_n( VECTOR_3D vP1, VECTOR_3D vP2, VECTOR_3D vP3, VECTOR_3D vP4 )
{
	PutTo( vP1, vP2, vP3, vP4 );
	P1.y *= -1.;
	P2.y *= -1.;
	P3.y *= -1.;
	P4.y *= -1.;
	Center();
}

void PANEL::PutTo_0( VECTOR_3D vP1, VECTOR_3D vP2, VECTOR_3D vP3, VECTOR_3D vP4 )
{
	PutTo( vP1, vP2, vP3, vP4 );
	P1.y = 0.;
	P2.y = 0.;
	P3.y = 0.;
	P4.y = 0.;
	Center();
}

void PANEL::Mirror( void )
{
	VECTOR_3D P;
	P = P1; P1 = P4; P4 = P;
	P = P2; P2 = P3; P3 = P;
	P1.y *= -1.;
	P2.y *= -1.;
	P3.y *= -1.;
	P4.y *= -1.;
	Center();
}

double PANEL::S( void )
{
	double SS2 = ((P1-P2)%(P3-P2)).Len();
	SS2 += ((P1-P4)%(P3-P4)).Len();
	if( SS2 < fabs(1.e-9) )SS2 = 0.;
	return 0.5 * SS2;
}

void PANEL::Center( void )
{
	Pc = 0.25*(P1 + P2 + P3 + P4);
}

void PANEL::Vout( FILE *ff )
{
	fprintf( ff, "% 12.7f % 12.7f % 12.7f", P1.x, P1.y, P1.z );
	fprintf( ff, "% 12.7f % 12.7f % 12.7f", P2.x, P2.y, P2.z );
	fprintf( ff, "% 12.7f % 12.7f % 12.7f", P3.x, P3.y, P3.z );
	fprintf( ff, "% 12.7f % 12.7f % 12.7f", P4.x, P4.y, P4.z );
}

void PANEL::Iout( FILE *ff )
{
	fprintf( ff, "%d %d %d %d", IdP1, IdP2, IdP3, IdP4 );
}

void PANEL::VoutR( FILE *ff )
{
	fprintf( ff, "% 12.7f % 12.7f % 12.7f", P4.x, -P4.y, P4.z );
	fprintf( ff, "% 12.7f % 12.7f % 12.7f", P3.x, -P3.y, P3.z );
	fprintf( ff, "% 12.7f % 12.7f % 12.7f", P2.x, -P2.y, P2.z );
	fprintf( ff, "% 12.7f % 12.7f % 12.7f", P1.x, -P1.y, P1.z );
}

void PANEL::ELset(const char *WingFus, char *Struct, int StructNr)
{
	sprintf(ElSet, "%s_%s_%i", WingFus, Struct, StructNr);
}
	
void PANEL::CheckType()
{
	if(IdP1 != IdP2)
	{
		if(IdP1 != IdP4)
		{
			if(IdP2 != IdP3)
			{
				if(IdP3 != IdP4)
				{
					Type = 0;			// quadrilateral panel
				}
				else Type = 4;			// triangle P3=P4 -> P4 to remove
			}
			else Type = 3;				// triangle P2=P3 -> P3 to remove
		}
		else Type = 4;					// triangle P1=P4 -> P4 to remove
	}
	else Type = 2;						// triangle P1=P2 -> P2 to remove
}		
