/*********************************************************************/
/*                                                                   */
/*                Panukl package - (C) TGG 2002                      */
/*             Config files library - (C) TGG 2015                   */
/*                                                                   */
/*********************************************************************/
/*                          Warszawa, 2011                           */
/*********************************************************************/
/*                                                                   */
/* File: panel.h                                                     */
/*                                                                   */
/* Author: T.Grabowski                                               */
/*                                                                   */
/* Contents - PANEL class - declaration                              */
/*                                                                   */
/* Last update: 6.04.2020                                            */
/*                                                                   */
/*********************************************************************/
/*                                                                   */

/** \file
    This file contains declaration of PANEL class
*/

#ifndef _PANEL_H_
#define _PANEL_H_

#include <cstring>
#include <stdio.h>
#include "vector3d.h"

/// A class for manipulating the rectangular panel

class PANEL
{
	public:
	
	PANEL( void );                                                        ///< default constructor
	PANEL( VECTOR_3D vP1, VECTOR_3D vP2, VECTOR_3D vP3, VECTOR_3D vP4 );  ///< constructor that sets corners from parameters
	~PANEL( void );
	
	char ElSet[256];                                                      ///< info for FEM analysis

	int Type;                                                             ///< type of panel: 0 - rectangle; 2, 3, 4 - triangle ( corner no. Type is doubled )

	int IdP1;                                                             ///< No. of the 1st corner of the panel
	int IdP2;                                                             ///< No. of the 2nd corner of the panel
	int IdP3;                                                             ///< No. of the 3rd corner of the panel
	int IdP4;                                                             ///< No. of the 4th corner of the panel
	
	int IdP5;                                                             ///< No. of the mid-point of the 1st side of the panel
	int IdP6;                                                             ///< No. of the mid-point of the 2nd side of the panel
	int IdP7;                                                             ///< No. of the mid-point of the 3rd side of the panel
	int IdP8;                                                             ///< No. of the mid-point of the 4th side of the panel
	
	VECTOR_3D P1;                                                         ///< the coordinates of the 1st corner of the panel
	VECTOR_3D P2;                                                         ///< the coordinates of the 2nd corner of the panel
	VECTOR_3D P3;                                                         ///< the coordinates of the 3rd corner of the panel
	VECTOR_3D P4;                                                         ///< the coordinates of the 4th corner of the panel
	
	VECTOR_3D Pc;                                                         ///< the coordinates of the central point of the panel
	
	VECTOR_3D *PP[4];                                                     ///< pointers to four corners (VECTOR_3D) of the panel
	double S( void );                                                     ///< returns the panel area
	void Center( void );                                                  ///< calculates the coordinates (Pc) of the central point of the panel
	
	void PutTo  ( VECTOR_3D vP1, VECTOR_3D vP2, VECTOR_3D vP3, VECTOR_3D vP4 );   ///< sets the corners (coordinates)
	void PutTo_n( VECTOR_3D vP1, VECTOR_3D vP2, VECTOR_3D vP3, VECTOR_3D vP4 );   ///< sets the corners (ccordinates) and changes the sign of y coordinate
	void PutTo_0( VECTOR_3D vP1, VECTOR_3D vP2, VECTOR_3D vP3, VECTOR_3D vP4 );   ///< sets the corners (ccordinates) and changes the coordinate to zero
	void Mirror( void );                                                          ///< makes a mirror with respect to xz plane, saves the order (CW or CCW) of corners
	
	void Clear( void );	                                                  ///< clears and assignes pointers to corners
	
	void Vout( FILE* ff );                                                ///< prints corners coordinates to stream ff
	void VoutR( FILE* ff );                                               ///< prints corners mirror (with respect to xz plane) coordinates to stream ff

	void Iout( FILE *ff );                                                ///< prints corners numbers to stream ff
	
	void ELset(const char *WingFus, char *Struct, int StructNr);          ///< sets panel info for FEM analysis to string ELSet
	void CheckType( void );                                               ///< checks the type of panel
};

#endif /*_PANEL_H_*/

