/*********************************************************************/
/*                                                                   */
/*             Config files library - (C) TGG 2018                   */
/*                                                                   */
/*********************************************************************/
/*                          Warszawa, 2018                           */
/*********************************************************************/
/*                                                                   */
/* File: panukl_acd.cpp                                              */
/*                                                                   */
/* Author: J.Mieloszyk, modif. T.Grabowski                           */
/*                                                                   */
/* Contents - acd file data (propeller info) class                   */
/*                                                                   */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/*                                                                   */

#include "panukl_acd.h"

using namespace std;

ACD_FILE::ACD_FILE( void ){ Clean (); }

ACD_FILE::~ACD_FILE( void ){}

void ACD_FILE::Clean( void )
{
    Prop.clear();
}

int ACD_FILE::Read(char *FileName) 
{
    ifstream in(FileName);

    if(in.fail())
        {
        fprintf( stderr, "[.ACD] file open error to reading (%s)\n", FileName );
        return (-1);
        }
    else
        {
        char Trush[256];
        int n_Props;
        PropDef PropData;

        in >> Trush >> n_Props;

        for(int x=0; x<n_Props; x++)
            {
            in >> PropData.Id;
            in >> PropData.X0;
            in >> PropData.Y0;
            in >> PropData.Z0;
            in >> PropData.Xthrust;
            in >> PropData.Ythrust;
            in >> PropData.Zthrust;
            in >> PropData.Omega;
            in >> PropData.D;
            in >> PropData.Dspinner;
            in >> PropData.DistribType;
            
            getline(in, PropData.Distrib);
            //cout << PropData.Distrib << endl;

            Prop.push_back(PropData);
            }
        }

    in.close();
    return 0;
}

int ACD_FILE::Write(char *FileName) 
{
    ofstream out(FileName);

    if(out.fail())
        {
        fprintf( stderr, "[.ACD] file open error to writing (%s)\n", FileName );
        return (-1);
        }
    else
    {
        out << "n_Props:\t" << Prop.size() << endl << endl;
        for(unsigned int x=0; x<Prop.size(); x++)
            {
            out << Prop[x].Id << "\t";
            out << Prop[x].X0 << "\t";
            out << Prop[x].Y0 << "\t";
            out << Prop[x].Z0 << "\t";
            out << Prop[x].Xthrust << "\t";
            out << Prop[x].Ythrust << "\t";
            out << Prop[x].Zthrust << "\t";
            out << Prop[x].Omega << "\t";
            out << Prop[x].D << "\t";
            out << Prop[x].Dspinner << "\t";
            out << Prop[x].DistribType << endl;
            out << Prop[x].Distrib << endl << endl;
            }
    }

    out.close();
    return 0;
}
