/*********************************************************************/
/*                                                                   */
/*             Config files library - (C) TGG 2018                   */
/*                                                                   */
/*********************************************************************/
/*                          Warszawa, 2018                           */
/*********************************************************************/
/*                                                                   */
/* File: panukl_acd.h                                                */
/*                                                                   */
/* Author: J.Mieloszyk, modif. T.Grabowski                           */
/*                                                                   */
/* Contents - acd file data (propeller info) class                   */
/*                                                                   */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/*                                                                   */

#ifndef _PANUKL_ACD_H_
#define _PANUKL_ACD_H_

#include <iostream>
#include <fstream>
#include <vector>

/// Structure to define actuator (propeller disc)

struct PropDef
{
    int    Id;                  ///< identification number
    double X0;                  ///< X coordinate of disc center
    double Y0;                  ///< Y coordinate of disc center
    double Z0;                  ///< Z coordinate of disc center
    double Xthrust;             ///< X component of thrust (for definition vector normal to actuator disc plane, components can be non-dimensional example: [1, 0, 0]) 
    double Ythrust;             ///< Y component of thrust
    double Zthrust;             ///< Z component of thrust
    double Omega;               ///< propeller angular rate in RPM, positive direction of rotation (right hand rule) defined around the thrust vector 
    double D;                   ///< propeller diameter 
    double Dspinner;            ///< propeller spinner diameter
    int    DistribType;         ///< type of velocity distribution Vprop/Vinfinity, induced by propeller, normal to actuator disc plane from propeller radius (0-parabolic, 1-elliptic, 2-from file)
    std::string Distrib;        ///< optional file containing the velocity distribution Vprop/Vinfinity, induced by propeller, normal to actuator disc plane from propeller radius (radius always scaled to unit size, values 0-1)
};

/// Parameters file to add propellers info to 3D panel code

class ACD_FILE 
{
    public:
    std::vector <PropDef> Prop;	///< vector of propellers

    ACD_FILE( void );
    ~ACD_FILE( void );
/// cleans local variables
    void Clean();
/// reads parameters file
    int Read( char *FileName );
/// writes parameters file
    int Write( char *Filename ); 

};
#endif /*_PANUKL_ACD_H_*/
