/*********************************************************************/ 
/*                                                                   */ 
/*             Config files library - (C) TGG 2015                   */ 
/*                                                                   */ 
/*********************************************************************/ 
/*                          Warszawa, 2015                           */ 
/*********************************************************************/ 
/*                                                                   */
/* File: panukl_con.cpp                                              */
/*                                                                   */
/* Author: T.Grabowski                                               */
/*                                                                   */
/* Contents - con file data (grid connection) class                  */
/*                                                                   */ 
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/*                                                                   */

#include "panukl_con.h"

CON_FILE::CON_FILE( void ){ Clean(); }

CON_FILE::~CON_FILE( void ){}

void CON_FILE::Clean( void )
{
    memset( DAT_file_1, 0, 256 );
    memset( DAT_file_2, 0, 256 );
    memset( DAT_file, 0, 256 );
	memset( CON_file, 0, 256 );

	for( int i=0; i<3; i++ )
        {
        dTrans1[i] = 0.;
        dTrans2[i] = 0.;
	    dRot1[i] = 0.;  
        dRot2[i] = 0.;  
	    dOrig1[i] = 0.; 
        dOrig2[i] = 0.;
        }
        
    iRotFlag1 = 0;    
    iRotFlag2 = 0;    
	
	dSref = 0.;     
    dBref = 0.;     
    dMACref = 0.;   
	dX25ref = 0.;   
    dZ25ref = 0.;   
	iFlagRef = 0;     
}

int CON_FILE::ReadCON( char *FileName )
{
	FILE *File;
	File = fopen( FileName, "r" );
	if( File == NULL )
		{
		fprintf( stderr, "[.CON] file open error to reading (%s)\n", FileName );
		return (-1);
		}
		
	//ReadComm( File );
	char cRob[256];

	ReadStr( File, DAT_file_1 );
	ClipFileName( DAT_file_1 );

	ReadVect( File, dTrans1, 3 );
	ReadVect( File, dRot1, 3 );
	ReadPar ( File, "%d", &iRotFlag1 );
	ReadVect( File, dOrig1, 3 );
	
	ReadStr( File, DAT_file_2 );
	ClipFileName( DAT_file_2 );

	ReadVect( File, dTrans2, 3 );
	ReadVect( File, dRot2, 3 );
	ReadPar ( File, "%d", &iRotFlag2 );
	ReadVect( File, dOrig2, 3 );
	
	ReadStr( File, DAT_file );
	ClipFileName( DAT_file );
	
	ReadPar ( File, "%d", &iFlagRef );
	ReadStr( File, cRob );
	sscanf( cRob, "%lf %lf %lf %lf %lf", &dSref, &dBref, &dMACref, &dX25ref, &dZ25ref );
	
	fclose( File );
	
	return (0);
}
	
int CON_FILE::WriteCON( char *FileName )
{
	FILE *File;
	File = fopen( FileName, "w" );
	if( File == NULL )
		{
		fprintf( stderr, "[.CON] file open error to writing (%s)\n", FileName );
		return (-1);
		}

	fprintf( File, "#	Connection program configuration file\n");
	fprintf( File, "%s	# master input data file [.DAT]\n", DAT_file_1 );
	fprintf( File, "%f %f %f # translation coordinates X Y Z (master)\n", dTrans1[0], dTrans1[1], dTrans1[2] );
	fprintf( File, "%f %f %f # rotation angles phi theta psi (master)\n", dRot1[0], dRot1[1], dRot1[2] );
	fprintf( File, "%3d                  # rotation origin option (0 user defined, 1 (0,0,0), 2 1/4 of MAC)\n", iRotFlag1 );
	fprintf( File, "%f %f %f # rotation origin coordinates (master)\n", dOrig1[0], dOrig1[1], dOrig1[2] );
	fprintf( File, "%s	# slave input data file [.DAT]\n", DAT_file_2 );
	fprintf( File, "%f %f %f # translation coordinates X Y Z (slave)\n", dTrans2[0], dTrans2[1], dTrans2[2] );
	fprintf( File, "%f %f %f # rotation angles phi theta psi (slave)\n", dRot2[0], dRot2[1], dRot2[2] );
	fprintf( File, "%3d                  # rotation origin option (0 user defined, 1 (0,0,0), 2 1/4 of MAC)\n", iRotFlag2 );
	fprintf( File, "%f %f %f # rotation origin coordinates (slave)\n", dOrig2[0], dOrig2[1], dOrig2[2] );
	fprintf( File, "%s	# output data file [.DAT]\n", DAT_file );
	fprintf( File, "%3d                  # reference values option (0 user defined, 1 master, 2 slave)\n", iFlagRef );
	fprintf( File, "%f %f %f %f %f # User defined reference values :S, B, MAC, X25, Z25\n",
	                    dSref, dBref, dMACref, dX25ref, dZ25ref );

	fclose( File );
	
	return (0);
}

