/*********************************************************************/ 
/*                                                                   */ 
/*             Config files library - (C) TGG 2018                   */ 
/*                                                                   */ 
/*********************************************************************/ 
/*                          Warszawa, 2018                           */ 
/*********************************************************************/ 
/*                                                                   */
/* File: panukl_con.h                                                */
/*                                                                   */
/* Author: T.Grabowski                                               */
/*                                                                   */
/* Contents - con file data (grid connection) class                  */
/*                                                                   */ 
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/*                                                                   */

#ifndef _PANUKL_CON_H_
#define _PANUKL_CON_H_

#include "iofun.h"

/// Parameters file for Connect program - grid connection

class CON_FILE : public IOFUN
{
	public:
	
	char DAT_file_1[256];               ///< 1st data (mesh+wake) file [.DAT] - input
	char DAT_file_2[256];               ///< 2nd data (mesh+wake) file [.DAT] - input
	
	char DAT_file[256];                 ///< connected data (mesh+wake) file [.DAT] - output
	
	double dTrans1[3];                  ///< translation of the 1st grid (3 coordinates)
    double dTrans2[3];                  ///< translation of the 2nd grid (3 coordinates)
	double dRot1[3];                    ///< rotation of the 1st grid (3 angles) [deg]
    double dRot2[3];                    ///< rotation of the 2nd grid (3 angles) [deg]
	double dOrig1[3];                   ///< origin of rotation of the 1st grid (3 coordinates)
    double dOrig2[3];                   ///< origin of rotation of the 2nd grid (3 coordinates)    
	int iRotFlag1;                      ///< 1st grid rotation origin option (0 user defined, 1 (0,0,0), 2 1/4 of MAC)
    int iRotFlag2;                      ///< 2nd grid rotation origin option (0 user defined, 1 (0,0,0), 2 1/4 of MAC)
	
	double dSref;                       ///< reference area defined by user
    double dBref;                       ///< reference wingspan defined by user
    double dMACref;                     ///< mean aerodynamic chord by user
	double dX25ref;                     ///< X coordinate of reference origin for moment calculation defined by user
    double dZ25ref;                     ///< Z coordinate of reference origin for moment calculation defined by user
	int iFlagRef;                       ///< reference values option (0 user defined, 1 master, 2 slave)
	
	char CON_file[256];                 ///< name of current [.con] file
	
	CON_FILE( void );
	~CON_FILE( void );
	
/// cleans local variables
	void Clean( void );
/// reads parameters [.con] file
	int ReadCON( char *FileName );
/// writes parameters [.con] file
	int WriteCON( char *Filename );
};

#endif /*_PANUKL_CON_H_*/
