/*********************************************************************/ 
/*                                                                   */ 
/*             Config files library - (C) TGG 2015                   */ 
/*                                                                   */ 
/*********************************************************************/ 
/*                          Warszawa, 2015                           */ 
/*********************************************************************/ 
/*                                                                   */
/* File: panukl_ngh.cpp                                              */
/*                                                                   */
/* Author: T.Grabowski                                               */
/*                                                                   */
/* Contents - ngh file data (wake and neighbours) class              */
/*                                                                   */ 
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/*                                                                   */

#include "panukl_ngh.h"

NGH_FILE::NGH_FILE( void ){ Clean(); }

NGH_FILE::~NGH_FILE( void ){}

void NGH_FILE::Clean( void )
{
	memset( DAT_file, 0, 256 );
	memset( INP_file, 0, 256 );
	memset( NGH_file, 0, 256 );

	dAlfa = 0.0;
	dBeta = 0.0;
	dAngleTE = 0.0;
	dAngleNGH = 0.0;
	dLastPanel = 0.0;
	iWakeType = 0;
}

int NGH_FILE::ReadNGH( char *FileName )
{
	FILE *File;
	File = fopen( FileName, "r" );
	if( File == NULL )
		{
		fprintf( stderr, "[.NGH] file open error to reading (%s)\n", FileName );
		return (-1);
		}
		
	ReadPar( File, "%d", &iWakeType );
	ReadPar( File, "%lf", &dAngleTE );
	ReadPar( File, "%lf", &dAngleNGH );
	ReadPar( File, "%lf", &dAlfa );
	ReadPar( File, "%lf", &dBeta );
	ReadPar( File, "%lf", &dLastPanel );
	ReadStr( File, INP_file );
	ReadStr( File, DAT_file );
	
	fclose( File );

	ClipFileName( INP_file );
	ClipFileName( DAT_file );
	
	strcpy( NGH_file, FileName );
	
	return (0);
}
	
int NGH_FILE::WriteNGH( char *FileName )
{
	FILE *File;
	File = fopen( FileName, "w" );
	if( File == NULL )
		{
		fprintf( stderr, "[.NGH] file open error to writing (%s)\n", FileName );
		return (-1);
		}

	fprintf( File, "#       Neigh program configuration file\n");
	fprintf( File, "%d		# wake type\n", iWakeType );
	fprintf( File, "%6.2f		# trailing edge angle\n", dAngleTE );
	fprintf( File, "%6.2f		# neighbour condition angle\n", dAngleNGH );
	fprintf( File, "%6.2f		# angle of attack\n", dAlfa );
	fprintf( File, "%6.2f		# sideslip angle\n", dBeta );
	fprintf( File, "%6.2f		# last panel length (MAC multiplication)\n", dLastPanel );
	fprintf( File, "%s	# input grid file\n", INP_file );
	fprintf( File, "%s	# output file - grid with wake and neighbours\n", DAT_file );

	fclose( File );
	
	return (0);
}
		
		
