/*********************************************************************/ 
/*                                                                   */ 
/*             Config files library - (C) TGG 2015                   */ 
/*                                                                   */ 
/*********************************************************************/ 
/*                          Warszawa, 2015                           */ 
/*********************************************************************/ 
/*                                                                   */
/* File: panukl_ngh.h                                                */
/*                                                                   */
/* Author: T.Grabowski                                               */
/*                                                                   */
/* Contents - ngh file data (wake and neighbours) class              */
/*                                                                   */ 
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/*                                                                   */

#ifndef _PANUKL_NGH_H_
#define _PANUKL_NGH_H_

#include "iofun.h"

/// Parameters file for Neigh program - wake and neighbours calculation

class NGH_FILE : public IOFUN
{
	public:
	
	char INP_file[256];   ///< data (mesh) file [.INP] - input
	char DAT_file[256];   ///< data (mesh+wake) file [.DAT] - output
	
	double dAlfa;         ///< angle of attack [deg] - active in case iWakeType = 1,3,4,6
	double dBeta;         ///< sideslip angle [deg] - active in case iWakeType = 2,3,5,6
	double dAngleTE;      ///< angle of trailing edge [deg], which determines wake creation (Kutta-Joukovsky condition)
	double dAngleNGH;     ///< max. value of angle [deg] (max. 90) between panels for which panels are treated as neighbouring
	double dLastPanel;    ///< length of wake - MAC multiplicator (default 20)
	
/** Type of wake:\n
 *  0 - wake paralel to MAC,\n
 *  1 - wake with downwash due to angle of attack,\n
 *  2 - wake with downwash due to sideslip angle,\n
 *  3 - wake parallel to the freestream,\n
 *  4-6 - wake with the same effect as in cases 1-3 but only for last wake panels.
 */
    int iWakeType;
	
	char NGH_file[256];   ///< name of current [.ngh] file
	
	NGH_FILE( void );
	~NGH_FILE( void );
	
/// cleans local variables
	void Clean( void );
/// reads parameters [.ngh] file
	int ReadNGH( char *FileName );
/// writes parameters [.ngh] file
	int WriteNGH( char *Filename );
	
};

#endif /*_PANUKL_NGH_H_*/
