/*********************************************************************/
/*                                                                   */
/*             Config files library - (C) TGG 2015                   */
/*                                                                   */
/*********************************************************************/
/*                          Warszawa, 2015                           */
/*********************************************************************/
/*                                                                   */
/* File: panukl_par.cpp                                              */
/*                                                                   */
/* Author: T.Grabowski                                               */
/*                                                                   */
/* Contents - par file data (doublet computation) class              */
/*                                                                   */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/*                                                                   */

#include "panukl_par.h"

PAR_FILE::PAR_FILE( void ){ Clean (); }

PAR_FILE::~PAR_FILE( void ){}

void PAR_FILE::Clean( void )
{
	memset( DAT_file, 0, 256 );
	memset( PAN_file, 0, 256 );
	memset( ACD_file, 0, 256 );

	dAlfa = 0.0;
	dBeta = 0.0;
	dP = dQ = dR = 0.0;
	iProc = 0;
	dMach = 0.0;
	iACD = 0;
}

int PAR_FILE::ReadPAR( char *FileName )
{
	FILE *File;
	File = fopen( FileName, "r" );
	if( File == NULL )
		{
		fprintf( stderr, "[.PAR] file open error to reading (%s)\n", FileName );
		return (-1);
		}
		
	ReadComm( File );
	ReadStr( File, DAT_file );
	ReadStr( File, PAN_file );
	ReadPar( File, "%lf", &dAlfa );
	ReadPar( File, "%lf", &dBeta );
	ReadPar( File, "%lf", &dP );
	ReadPar( File, "%lf", &dQ );
	ReadPar( File, "%lf", &dR );
	ReadPar( File, "%d", &iProc );
	if(ReadPar( File, "%lf", &dMach )==EOF)dMach=0.;
	if(ReadPar( File, "%d", &iACD )==EOF)iACD=0;
	if(iACD)ReadStr( File, ACD_file );
	
	fclose( File );

	ClipFileName( PAN_file );
	ClipFileName( DAT_file );
	ClipFileName( ACD_file );
	
	strcpy( PAR_file, FileName );
	
	return (0);
}
	
int PAR_FILE::WritePAR( char *FileName )
{
	FILE *File;
	File = fopen( FileName, "w" );
	if( File == NULL )
		{
		fprintf( stderr, "[.PAR] file open error to writing (%s)\n", FileName );
		return (-1);
		}

	fprintf( File, "#	Panukl main program configuration file\n");
	fprintf( File, "%s	# input data file (grid with wake)\n", DAT_file );
	fprintf( File, "%s	# output file (input for Press program)\n", PAN_file );
	fprintf( File, "%6.2f		# alfa - angle of attack [deg]\n", dAlfa );
	fprintf( File, "%6.2f		# beta - sideslip angle [deg]\n", dBeta );
	fprintf( File, "%6.2f		# P - roll rate [rad/s]\n", dP );
	fprintf( File, "%6.2f		# Q - pitch rate [rad/s]\n", dQ );
	fprintf( File, "%6.2f		# R - yaw rate [rad/s]\n", dR );
	fprintf( File, "%3d		# type of linear system solver (1-LAPACK OPTIMIZED, 0-NOT OPTIMIZED)\n", iProc );
	fprintf( File, "%6.3f		# Mach number\n", dMach );
	fprintf( File, "%3d		# ACD file flag\n", iACD );
	if(iACD)fprintf( File, "%s	# propeller data configuration file (input)\n", ACD_file );


	fclose( File );
	
	return (0);
}
		
