/*********************************************************************/
/*                                                                   */
/*             Config files library - (C) TGG 2015                   */
/*                                                                   */
/*********************************************************************/
/*                          Warszawa, 2015                           */
/*********************************************************************/
/*                                                                   */
/* File: panukl_par.h                                                */
/*                                                                   */
/* Author: T.Grabowski                                               */
/*                                                                   */
/* Contents - par file data (doublet computation) class              */
/*                                                                   */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/*                                                                   */

#ifndef _PANUKL_PAR_H_
#define _PANUKL_PAR_H_

#include "iofun.h"

/// Parameters file for Panukl program (computation of velocity potential distribution)

class PAR_FILE : public IOFUN
{
    public:

    char DAT_file[256];    ///< input file - mesh + wake data file [.DAT]
    char PAN_file[256];    ///< output file - doublet distribution file [.PAN]
    char ACD_file[256];    ///< optional input file - propellers data configuration file [.ACD]

    double dAlfa;          ///< angle of attack [deg]
    double dBeta;          ///< sideslip angle [deg]
    double dMach;          ///< Mach number	
    double dP;             ///< roll rate [rad/s]
    double dQ;             ///< pitch rate [rad/s]
    double dR;             ///< yaw rate [rad/s]
    int iProc;             ///< type of linear system solver (1-LAPACK OPTIMIZED, 0-NOT OPTIMIZED)
    int iACD;              ///< flag of using the propellers data configuration file (0-no file, 1-file present)
    char PAR_file[256];    ///< current parameters file [.PAR]

    PAR_FILE( void );
    ~PAR_FILE( void );
/// cleans local variables	
    void Clean();
/// reads parameters file
    int ReadPAR( char *FileName );
/// writes parameters file
    int WritePAR( char *Filename ); 

};

#endif /*_PANUKL_PAR_H_*/
