/*********************************************************************/ 
/*                                                                   */ 
/*             Config files library - (C) TGG 2019                   */ 
/*                                                                   */ 
/*********************************************************************/ 
/*                          Warszawa, 2019                           */ 
/*********************************************************************/ 
/*                                                                   */
/* File: panukl_plr.h                                                */
/*                                                                   */
/* Author: T.Grabowski                                               */
/*                                                                   */
/* Contents - plr file data (POLAR creation) class                   */
/*                                                                   */ 
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/*                                                                   */


#include "panukl_plr.h"

PANUKL_PLR::PANUKL_PLR( void ){ Clean(); }

PANUKL_PLR::~PANUKL_PLR( void ){}

void PANUKL_PLR::Clean( void )
{
	memset( POLAR_dir, 0, 256 );
	memset( DAT_file, 0, 256 );
	memset( OUT_file, 0, 256 );

	dAlfa0 = dAlfak = dAlf = 0.0;
	dMa = 0.0;
	iNum1 = iNum2 = 0;
	iProc = 0;
	iMeth = 0;
	iAngle = 0;
	iCompress = 0;
	iPitchX = 0;
	dXNum1 = dXNum2 = 0.0;
	dCfe = 0.0;
	iMaxProc = 0;
}

int PANUKL_PLR::Read( char *PolarFile )
{
	FILE *File;
	File = fopen( PolarFile, "r" );
	if( File == NULL )
		{
		fprintf( stderr, "[.PLR] file open error to reading (%s)\n", PolarFile );
		return (-1);
		}
		
	ReadComm( File );
	ReadStr( File, POLAR_dir );
	ReadStr( File, DAT_file );
	ReadStr( File, OUT_file );
	ReadPar( File, "%lf %lf %lf", &dAlfa0, &dAlfak, &dAlf );
	ReadPar( File, "%lf", &dMa );

	ReadPar( File, "%d", &iProc );
	
	ReadPar( File, "%d %d", &iNum1, &iNum2 );
	ReadPar( File, "%d", &iMeth );
	ReadPar( File, "%d", &iAngle );
	ReadPar( File, "%lf %lf", &dXNum1, &dXNum2 ); 
	ReadPar( File, "%d", &iCompress );
	ReadPar( File, "%d", &iPitchX );
	
	ReadPar( File, "%lf", &dCfe );

	ReadPar( File, "%d", &iMaxProc );
	
	fclose( File );

	ClipFileName( POLAR_dir );
	ClipFileName( DAT_file );
	ClipFileName( OUT_file );
	
	iMaxProc = iMaxProc > 64 ? 64 : iMaxProc;
	iMaxProc = iMaxProc < 1 ? 1 : iMaxProc;
	
	return (0);
}

int PANUKL_PLR::Write( char *PolarFile )
{
	FILE *File;
	File = fopen( PolarFile, "w" );
	if( File == NULL )
		{
		fprintf( stderr, "[.PLR] file open error to writing (%s)\n", PolarFile );
		return (-1);
		}

	fprintf( File, "#	Panukl -> POLAR configuration file\n");
	fprintf( File, "%s	# POLAR output directory\n", POLAR_dir );
	fprintf( File, "%s	# data (grid+wake) file [.dat] for A/C configuration\n", DAT_file );
	fprintf( File, "%s	# output polar file [.TXT]\n", OUT_file );

	fprintf( File, "%7.3f %7.3f %7.3f	# Alfa0, Alfa1, dA - angle of attack sequence [deg]\n", dAlfa0, dAlfak, dAlf );
	fprintf( File, "%7.3f               # Mach number \n", dMa );

	fprintf( File, "%3d		# type of linear system solver (1-LAPACK OPTIMIZED, 0-NOT OPTIMIZED)\n", iProc );

	fprintf( File, "%d %d		# range of panel's indices for pressure calculation\n", iNum1, iNum2 );
	fprintf( File, "%3d		# number of differentiation method\n", iMeth );
	fprintf( File, "%3d		# averaging flag\n", iAngle );
	fprintf( File, "%6.2f %6.2f	# range of X coordinates for pressure calculation\n", dXNum1, dXNum2 ); 
	fprintf( File, "%3d		# compressibility flag (0-none, 1-PG, 2-KT, 3-direct)\n", iCompress );
	fprintf( File, "%3d		# X component of pressure in pitching moment\n", iPitchX );

	fprintf( File, "%10.6f		# Cfe - equivalent parasite drag\n", dCfe );

	fprintf( File, "%3d		# max number of parallel process\n", iMaxProc );

	fclose( File );
	
	return (0);
}
