/*********************************************************************/ 
/*                                                                   */ 
/*             Config files library - (C) TGG 2019                   */ 
/*                                                                   */ 
/*********************************************************************/ 
/*                          Warszawa, 2019                           */ 
/*********************************************************************/ 
/*                                                                   */
/* File: panukl_plr.h                                                */
/*                                                                   */
/* Author: T.Grabowski                                               */
/*                                                                   */
/* Contents - plr file data (POLAR creation) class                   */
/*                                                                   */ 
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/*                                                                   */

#ifndef _PANUKL_PLR_H_
#define _PANUKL_PLR_H_

#include "iofun.h"

/// Parameters file for PanProject program - creates POLAR for particular A/C configuration (grid+wake file)

class PANUKL_PLR : public IOFUN
{
	public:

	char POLAR_dir[256];   ///< path name for output - aerodynamic data files to create POLAR

	double dAlfa0;         ///< left (lower) bound of angle of attack [deg]
	double dAlfak;         ///< right (upper) bound of angle of attack [deg]
	double dAlf;           ///< step of angle of attack [deg]
	double dMa;            ///< Mach number

	double dCfe;           ///< parasite drag coefficient

	char DAT_file[256];    ///< data (grid+wake) file [.DAT] for A/C configuration
	char OUT_file[256];    ///< output polar file [.TXT]

	int iCompress;         ///< compressibility flag (0-none, 1-Prandtl-Galuert, 2-Karman-Tsien, 3-direct)
	int iMaxProc;          ///< max number of parallel process (shouldn't be greater than real number of CPUs (cores)
	int iProc;             ///< current number of CPUs (cores) - set by program
	                       
	int iNum1;             ///< number of the first panel taken for global coefficients calculation
	int iNum2;             ///< number of the last panel taken for global coefficients calculation
	int iMeth;             ///< number of differentiation method (see PANUKL manual)
	int iAngle;            ///< averaging flag (0/1)
	int iPitchX;           ///< flag to take X component of forces for pitching moment calculation(0/1)
	                       
	double dXNum1;         ///< lower bound of range of X coordinate taken for global coefficients calculation
	double dXNum2;	       ///< upper bound of range of X coordinate taken for global coefficients calculation	

	PANUKL_PLR( void );
	~PANUKL_PLR( void );

/// cleans local variables
	void Clean();
/// reads the [.plr] file
	int Read( char *PolarFile );
/// writes the [.plr] file
	int Write( char *PolarFile );
};

#endif /*_PANUKL_PLR_H_*/
