/*********************************************************************/ 
/*                                                                   */ 
/*             Config files library - (C) TGG 2015                   */ 
/*                                                                   */ 
/*********************************************************************/ 
/*                          Warszawa, 2015                           */ 
/*********************************************************************/ 
/*                                                                   */
/* File: panukl_prj.cpp                                              */
/*                                                                   */
/* Author: T.Grabowski                                               */
/*                                                                   */
/* Contents - prj file data (SDSA creation) class                    */
/*                                                                   */ 
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/*                                                                   */

#include "panukl_prj.h"

PANUKL_PRJ::PANUKL_PRJ( void ){ Clean(); }

PANUKL_PRJ::~PANUKL_PRJ( void ){}

void PANUKL_PRJ::Clean( void )
{
	memset( SDSA_dir, 0, 256 );
	memset( DAT_file, 0, 256 );
	memset( DAT_file1, 0, 256 );
	memset( DAT_file2, 0, 256 );
	memset( DAT_file3, 0, 256 );
	
	iH = iV = iL = 0;
	dH = dV = dL = 0.0;
	dAlfa0 = dAlfak = dAlf = 0.0;
	dMa0 = dMak = dMa = 0.0;
	dBeta = 0.0;
	dP = dQ = dR = 0.0;
	iNum1 = iNum2 = 0;
	iProc = 0;
	iMeth = 0;
	iAngle = 0;
	iCompress = 0;
	iPitchX = 0;
	dXNum1 = dXNum2 = 0.0;
	dCfe = 0.0;
	dAoA_max = 0.0;
	iMaxProc = 0;
}

int PANUKL_PRJ::Read( char *PrjFile )
{
	FILE *File;
	File = fopen( PrjFile, "r" );
	if( File == NULL )
		{
		fprintf( stderr, "[.PRJ] file open error to reading (%s)\n", PrjFile );
		return (-1);
		}
		
	ReadComm( File );
	ReadStr( File, SDSA_dir );
	ReadStr( File, DAT_file );
	ReadStr( File, DAT_file1 );
	ReadStr( File, DAT_file2 );
	ReadStr( File, DAT_file3 );
	ReadPar( File, "%d %d %d", &iH, &iL, &iV );
	ReadPar( File, "%lf %lf %lf", &dH, &dL, &dV );
	ReadPar( File, "%lf %lf %lf", &dAlfa0, &dAlfak, &dAlf );
	ReadPar( File, "%lf %lf %lf", &dMa0, &dMak, &dMa );
	ReadPar( File, "%lf", &dBeta );
	ReadPar( File, "%lf", &dP );
	ReadPar( File, "%lf", &dQ );
	ReadPar( File, "%lf", &dR );
	ReadPar( File, "%d", &iProc );
	
	ReadPar( File, "%d %d", &iNum1, &iNum2 );
	ReadPar( File, "%d", &iMeth );
	ReadPar( File, "%d", &iAngle );
	ReadPar( File, "%lf %lf", &dXNum1, &dXNum2 ); 
	ReadPar( File, "%d", &iCompress );
	ReadPar( File, "%d", &iPitchX );
	
	ReadPar( File, "%lf", &dCfe );
	ReadPar( File, "%lf", &dAoA_max );

	ReadPar( File, "%d", &iMaxProc );
	
	fclose( File );

	ClipFileName( SDSA_dir );
	ClipFileName( DAT_file );
	ClipFileName( DAT_file1 );
	ClipFileName( DAT_file2 );
	ClipFileName( DAT_file3 );
	
	iMaxProc = iMaxProc > 64 ? 64 : iMaxProc;
	iMaxProc = iMaxProc < 1 ? 1 : iMaxProc;
	
	return (0);
}

int PANUKL_PRJ::Write( char *PrjFile )
{
	FILE *File;
	File = fopen( PrjFile, "w" );
	if( File == NULL )
		{
		fprintf( stderr, "[.PRJ] file open error to writing (%s)\n", PrjFile );
		return (-1);
		}

	fprintf( File, "#	Panukl -> SDSA configuration file\n");
	fprintf( File, "%s	# SDSA project directory\n", SDSA_dir );
	fprintf( File, "%s	# clean configuration data file [.dat] (grid with wake)\n", DAT_file );
	fprintf( File, "%s	# Elevator deflected configuration data file [.dat] (grid with wake)\n", DAT_file1 );
	fprintf( File, "%s	# Aileron deflected configuration data file [.dat] (grid with wake)\n", DAT_file2 );
	fprintf( File, "%s	# Rudder deflected configuration data file [.dat] (grid with wake)\n", DAT_file3 );
	fprintf( File, "%d %d %d # deflected configuration flags\n", iH, iL, iV );
	fprintf( File, "%f %f %f # control surfaces deflection\n", dH, dL, dV );

	fprintf( File, "%7.3f %7.3f %7.3f	# Alfa0, Alfa1, dA - angle of attack sequence [deg]\n", dAlfa0, dAlfak, dAlf );
	fprintf( File, "%7.3f %7.3f %7.3f	# Mach0, Mach1, dM - Mach number sequence \n", dMa0, dMak, dMa );
	fprintf( File, "%7.3f		# beta - sideslip angle [deg]\n", dBeta );
	fprintf( File, "%6.2f		# P - roll rate [rad/s]\n", dP );
	fprintf( File, "%6.2f		# Q - pitch rate [rad/s]\n", dQ );
	fprintf( File, "%6.2f		# R - yaw rate [rad/s]\n", dR );
	fprintf( File, "%3d		# type of linear system solver (1-LAPACK OPTIMIZED, 0-NOT OPTIMIZED)\n", iProc );

	fprintf( File, "%d %d		# range of panel's indices for pressure calculation\n", iNum1, iNum2 );
	fprintf( File, "%3d		# number of differentiation method\n", iMeth );
	fprintf( File, "%3d		# averaging flag\n", iAngle );
	fprintf( File, "%6.2f %6.2f	# range of X coordinates for pressure calculation\n", dXNum1, dXNum2 ); 
	fprintf( File, "%3d		# compressibility flag (0-none, 1-PG, 2-KT, 3-direct)\n", iCompress );
	fprintf( File, "%3d		# X component of pressure in pitching moment\n", iPitchX );

	fprintf( File, "%10.6f		# Cfe - equivalent parasite drag\n", dCfe );
	fprintf( File, "%10.3f		# AoA max - estimated max. angle of attack [deg]\n", dAoA_max );

	fprintf( File, "%3d		# max number of parallel process\n", iMaxProc );

	fclose( File );
	
	return (0);
}
