/*********************************************************************/ 
/*                                                                   */ 
/*             Config files library - (C) TGG 2015                   */ 
/*                                                                   */ 
/*********************************************************************/ 
/*                          Warszawa, 2015                           */ 
/*********************************************************************/ 
/*                                                                   */
/* File: panukl_prj.h                                                */
/*                                                                   */
/* Author: T.Grabowski                                               */
/*                                                                   */
/* Contents - prj file data (SDSA creation) class                    */
/*                                                                   */ 
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/*                                                                   */

#ifndef _PANUKL_PRJ_H_
#define _PANUKL_PRJ_H_

#include "iofun.h"

/// Parameters file for PanProject program - creates data set for SDSA

class PANUKL_PRJ : public IOFUN
{
	public:

	char SDSA_dir[256];    ///< path name for output - aerodynamic data files set for SDSA
	int iH;                ///< flag to compute configuration with elevator deflected (0/1)
	int iL;                ///< flag to compute configuration with aileron deflected (0/1)
	int iV;                ///< flag to compute configuration with rudder deflected (0/1)

	double dAlfa0;         ///< left (lower) bound of angle of attack [deg]
	double dAlfak;         ///< right (upper) bound of angle of attack [deg]
	double dAlf;           ///< step of angle of attack [deg]
	double dBeta;          ///< sideslip angle [deg]
	double dP;             ///< roll rate [rad/s]		
	double dQ;             ///< pitch rate [rad/s]		
	double dR;             ///< yaw rate [rad/s]		
	double dMa0;           ///< left (lower) bound of Mach number range
	double dMak;           ///< right (upper) bound of Mach number range
	double dMa;            ///< step of Mach number
	double dH;             ///< elevator deflection corresponding to mesh from DAT_file1 [deg]
	double dL;             ///< aileron deflection corresponding to mesh from DAT_file2 [deg]
	double dV;             ///< rudder deflection corresponding to mesh from DAT_file3 [deg]

	double dCfe;           ///< parasite drag coefficient
	double dAoA_max;       ///< max. value of angle of attack [deg]

	char DAT_file[256];    ///< data (mesh+wake) file [.DAT] for basic configuration
	char DAT_file1[256];   ///< data (mesh+wake) file [.DAT] elevator deflected
	char DAT_file2[256];   ///< data (mesh+wake) file [.DAT] aileron deflected
	char DAT_file3[256];   ///< data (mesh+wake) file [.DAT] rudder deflected

	int iCompress;         ///< compressibility flag (0-none, 1-Prandtl-Galuert, 2-Karman-Tsien, 3-direct)
	int iMaxProc;          ///< max number of parallel process (shouldn't be greater than real number of CPUs (cores)
	int iProc;             ///< current number of CPUs (cores) - set by program
	                       
	int iNum1;             ///< number of the first panel taken for global coefficients calculation
	int iNum2;             ///< number of the last panel taken for global coefficients calculation
	int iMeth;             ///< number of differentiation method (see PANUKL manual)
	int iAngle;            ///< averaging flag (0/1)
	int iPitchX;           ///< flag to take X component of forces for pitching moment calculation(0/1)
	                       
	double dXNum1;         ///< lower bound of range of X coordinate taken for global coefficients calculation
	double dXNum2;	       ///< upper bound of range of X coordinate taken for global coefficients calculation	

	PANUKL_PRJ( void );
	~PANUKL_PRJ( void );

/// cleans local variables
	void Clean();
/// reads the [.prj] file
	int Read( char *PrjFile );
/// writes the [.prj] file
	int Write( char *PrjFile );
};

#endif /*_PANUKL_PRJ_H_*/
