/*********************************************************************/ 
/*                                                                   */ 
/*             Config files library - (C) TGG 2015                   */ 
/*                                                                   */ 
/*********************************************************************/ 
/*                          Warszawa, 2015                           */ 
/*********************************************************************/ 
/*                                                                   */
/* File: panukl_prs.cpp                                              */
/*                                                                   */
/* Author: T.Grabowski                                               */
/*                                                                   */
/* Contents - prs file data (pressure distribution) class            */
/*                                                                   */ 
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/*                                                                   */

#include "panukl_prs.h"

PRS_FILE::PRS_FILE( void ){ Clean(); }

PRS_FILE::~PRS_FILE( void ){}

void PRS_FILE::Clean( void )
{
	memset( PAN_file, 0, 256 );
	memset( TXT_file, 0, 256 );
	memset( CZY_file, 0, 256 );
	memset( DAT_file, 0, 256 );
	memset( OUT_file, 0, 256 );
	memset( BLN_file, 0, 256 );
	memset( EPS_file, 0, 256 );
	memset( PRS_file, 0, 256 );
	memset( SDS_file, 0, 256 );
	
	iEps = 0;
	iInd = 0;
	iMach = 0;
	iNXY_Eps = iNZ_Eps = 0;
	iNum1 = iNum2 = 0;
	iMeth = 0;
	iAngle = 0;
	iPitchX = 0;
	iOutFiles = 0;
	
	dXEps = 0.0;
	dYX1_Eps = dYX2_Eps = 0.0;
	dZ1_Eps = dZ2_Eps = 0.0;
	dXNum1 = dXNum2 = 0.0;
	dMach = 0.0;
}

int PRS_FILE::ReadPRS( const char *FileName )
{
	Clean();
	
	FILE *File;
	File = fopen( FileName, "r" );
	if( File == NULL )
		{
		fprintf( stderr, "[.PRS] file open error to reading (%s)\n", FileName );
		return (-1);
		}

	ReadComm( File );
	ReadStr( File, PAN_file );
	ClipFileName( PAN_file );
	ReadPar( File, "%d", &iEps );
	ReadPar( File, "%d", &iInd );
	ReadPar( File, "%d %d", &iNXY_Eps, &iNZ_Eps );
	ReadPar( File, "%lf", &dXEps );
	ReadPar( File, "%lf %lf", &dYX1_Eps, &dYX2_Eps );
	ReadPar( File, "%lf %lf", &dZ1_Eps, &dZ2_Eps );
	ReadPar( File, "%d %d", &iNum1, &iNum2 );
	ReadPar( File, "%d", &iMeth );
	ReadPar( File, "%d", &iAngle );
	ReadPar( File, "%lf %lf", &dXNum1, &dXNum2 ); 
	if(ReadPar( File, "%lf", &dMach )==EOF)dMach=0.;
	if(ReadPar( File, "%d", &iMach )==EOF)iMach=0;
	if(ReadPar( File, "%d", &iPitchX )==EOF)iPitchX=0;
	if(ReadPar( File, "%d", &iOutFiles )==EOF)iOutFiles=0;
	if(iOutFiles)
		{
		ReadStr( File, TXT_file );
		ReadStr( File, OUT_file );
		ReadStr( File, CZY_file );
		ReadStr( File, BLN_file );
		ReadStr( File, EPS_file );
		ReadStr( File, SDS_file );
		ClipFileName( TXT_file );
		ClipFileName( OUT_file );
		ClipFileName( CZY_file );
		ClipFileName( BLN_file );
		ClipFileName( EPS_file );
		ClipFileName( SDS_file );
		}

	fclose( File );

	strcpy( PRS_file, FileName );

	return (0);
}
	
int PRS_FILE::WritePRS( const char *FileName )
{
	FILE *File;
	File = fopen( FileName, "w" );
	if( File == NULL )
		{
		fprintf( stderr, "[.PRS] file open error to writing (%s)\n", FileName );
		return (-1);
		}

	fprintf( File, "#	Press program configuration file\n");
	fprintf( File, "%s	# input data file [.PAN]\n", PAN_file );
	fprintf( File, "%3d		# downwash calculation flag\n", iEps );
	fprintf( File, "%3d		# induced drag calculation flag\n", iInd );
	fprintf( File, "%d %d		# range of panel's indices for downwash calculation\n", iNXY_Eps, iNZ_Eps );
	fprintf( File, "%6.2f		# X coordinate of downwash plane\n", dXEps );
	fprintf( File, "%6.2f %6.2f	# Y (X) boundary coordinates for downwash calculation\n", dYX1_Eps, dYX2_Eps );
	fprintf( File, "%6.2f %6.2f	# Z boundary coordinates for downwash calculation\n", dZ1_Eps, dZ2_Eps );
	fprintf( File, "%d %d		# range of panel's indices for pressure calculation\n", iNum1, iNum2 );
	fprintf( File, "%3d		# number of differentiation method\n", iMeth );
	fprintf( File, "%3d		# averaging flag\n", iAngle );
	fprintf( File, "%6.2f %6.2f	# range of X coordinates for pressure calculation\n", dXNum1, dXNum2 ); 
	fprintf( File, "%6.2f		# Mach number for correction\n", dMach );
	fprintf( File, "%3d		# compressibility correction flag (0-none, 1-PG, 2-KT)\n", iMach );
	fprintf( File, "%3d		# X component of pressure in pitching moment\n", iPitchX );
	fprintf( File, "%3d		# output files names flag (0-automatic, 1-user defined)\n", iOutFiles );
	if(iOutFiles)
		{
		fprintf( File, "%s	# output file - pressure distribution [.txt] and other results\n", TXT_file );
		fprintf( File, "%s	# output file - aerodynamic forces and moments coefficients & other global results [.out]\n", OUT_file );
		fprintf( File, "%s	# output file - wingspan loads distribution [.czy]\n", CZY_file );
		fprintf( File, "%s	# output file - contour of cross section [.bln]\n", BLN_file );
		fprintf( File, "%s	# output file - velocity and downwash distribution [.eps]\n", EPS_file );
		fprintf( File, "%s	# output file - the same as [.out] in SDSA format [.sdsa]\n", SDS_file );
		}

	fclose( File );

	return (0);
}

void PRS_FILE::DefFile( char *cRob, char *cFile, const char *cExt )
{
	if( cFile[0] == 0 )
		{
		strcpy( cFile, cRob );
		filename_setext( cFile, sizeof(cFile), cExt );
		}
}

void PRS_FILE::DefNames( char *OUTdir )
{
	char cRob[256];
	strcpy( cRob, OUTdir );
	strcat( cRob, filename_name( (const char*)PRS_file ) );

	DefFile( cRob, TXT_file, ".txt" );
	DefFile( cRob, OUT_file, ".out" );
	DefFile( cRob, CZY_file, ".czy" );
	DefFile( cRob, EPS_file, ".eps" );
	DefFile( cRob, BLN_file, ".bln" );
	DefFile( cRob, SDS_file, ".sdsa" );
}
