/*********************************************************************/ 
/*                                                                   */ 
/*             Config files library - (C) TGG 2015                   */ 
/*                                                                   */ 
/*********************************************************************/ 
/*                          Warszawa, 2015                           */ 
/*********************************************************************/ 
/*                                                                   */
/* File: panukl_prs.h                                                */
/*                                                                   */
/* Author: T.Grabowski                                               */
/*                                                                   */
/* Contents - prs file data (pressure distribution) class            */
/*                                                                   */ 
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/*                                                                   */

#ifndef _PANUKL_PRS_H_
#define _PANUKL_PRS_H_

#include "iofun.h"

/// Parameters file for Press program (computation of pressure distribution)

class PRS_FILE : public IOFUN
{
	public:

	char PAN_file[256];  ///< input file - doublet distribution file [.PAN]
	char DAT_file[256];  ///< input file - mesh + wake data file [.DAT]	
	char TXT_file[256];  ///< output file - pressure distribution [.txt] and other results
	char OUT_file[256];  ///< output file - aerodynamic forces and moments coefficients & other global results [.out]
	char CZY_file[256];  ///< output file - spanwise loads distribution [.czy]
	char BLN_file[256];  ///< output file - contour of cross section [.bln]
	char EPS_file[256];  ///< output file - velocity and downwash distribution [.eps]
	char SDS_file[256];  ///< output file - the same as [.out] in SDSA format [.sdsa]

	int iEps;            ///< calculation of velocity and downwash distribution - flag (0-none,1-YZ plane,2-XZ plane)
	int iInd;            ///< calculation of induced drag - flag (0,1)
	int iMach;           ///< compressibility flag (0-none, 1-Prandtl-Galuert, 2-Karman-Tsien) - not active in case compressibility was set for doublet distribution
	int iNXY_Eps;        ///< number of mesh nodes for velocity field calculation longwise X (iEps=2) or Y (iEps=1) axis
	int iNZ_Eps;         ///< number of mesh nodes for velocity field calculation longwise Z
	int iNum1;           ///< number of the first panel taken for global coefficients calculation
	int iNum2;           ///< number of the last panel taken for global coefficients calculation
	int iMeth;           ///< number of differentiation method (see PANUKL manual)
	int iAngle;          ///< averaging flag (0/1)
	int iPitchX;         ///< flag to take X component of forces for pitching moment calculation(0/1)
	int iOutFiles;       ///< flag to control output files names: 0 - automatic names derived from PRS_file, 1 - set by user

	double dXEps;        ///< X coordinate (or Y coordinate in case iEps=2) of plane where velocity field is calculated
	double dYX1_Eps;     ///< Y coordinate of left bound (or X in case iEps=2) of plane where velocity field is calculated
	double dYX2_Eps;     ///< Y coordinate of right bound (or X in case iEps=2) of plane where velocity field is calculated
	double dZ1_Eps;      ///< Z coordinate of lower bound of plane where velocity field is calculated
	double dZ2_Eps;      ///< Z coordinate of upper bound of plane where velocity field is calculated
	double dXNum1;       ///< lower bound of range of X coordinate taken for global coefficients calculation
	double dXNum2;       ///< upper bound of range of X coordinate taken for global coefficients calculation
	double dMach;        ///< Mach number to take into account compressibility effect - not active in case compressibility was set for doublet distribution

	char PRS_file[256];  ///< name of current [.prs] file

	PRS_FILE( void );
	~PRS_FILE( void );

/// cleans local variables
	void Clean( void );
/// reads the [.prs] file
	int ReadPRS( const char *FileName );
/// writes the [.prs] file
	int WritePRS( const char *Filename );
/// set default names of output files
	void DefNames( char *OUTdir );
/// set default name if it is empty
	void DefFile( char *cRob, char *cFile, const char *cExt );

};

#endif /*_PANUKL_PRS_H_*/
