/*********************************************************************/ 
/*                                                                   */ 
/*             Config files library - (C) TGG 2015                   */ 
/*                                                                   */ 
/*********************************************************************/ 
/*                          Warszawa, 2018                           */ 
/*********************************************************************/ 
/*                                                                   */
/* File: sdsa_file.cpp                                               */
/*                                                                   */
/* Author: T.Grabowski                                               */
/*                                                                   */
/* Contents - sdsa file (simple output file) class definition        */
/*                                                                   */ 
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/*                                                                   */

#include "sdsa_file.h"

SDSA_FILE::SDSA_FILE( void ){ Clean(); }

SDSA_FILE::~SDSA_FILE( void ){}

void SDSA_FILE::Clean( void )
{
	S = B = MAC = Swet = 0.0;
	X = Z = 0.0;
	Alfa = Beta = Mach = 0.0;
	P = Q = R = 0.0;
	Cx = Cy = Cz = Cl = Cm = Cn = 0.0;
	Cxs = Cys = Czs = Cls = Cms = Cns = 0.0;
	Cxi = Czi = 0.0;
	
	file_flag = 0;
}

void SDSA_FILE::SetName( const char* SDSA_file )
{
	strcpy( file_name, SDSA_file );
	file_flag = 1;
}

int SDSA_FILE::Read( const char* SDSA_file )
{
	FILE *File;
	File = fopen( SDSA_file, "r" );
	if( File == NULL )
		{
		fprintf( stderr, "[.sdsa] file open error to reading (%s)\n", SDSA_file );
		return (-1);
		}
		
	ReadPar( File, "%lf", &S );
	ReadPar( File, "%lf", &MAC );
	ReadPar( File, "%lf", &B );
	ReadPar( File, "%lf", &Swet );
	ReadPar( File, "%lf %lf", &X, &Z );
	ReadPar( File, "%lf", &Alfa );
	ReadPar( File, "%lf", &Beta );
	ReadPar( File, "%lf", &Mach );
	ReadPar( File, "%lf", &P );
	ReadPar( File, "%lf", &Q );
	ReadPar( File, "%lf", &R );
	ReadPar( File, "%lf", &Cx );
	ReadPar( File, "%lf", &Cy );
	ReadPar( File, "%lf", &Cz );
	ReadPar( File, "%lf", &Cl );
	ReadPar( File, "%lf", &Cm );
	ReadPar( File, "%lf", &Cn );
	ReadPar( File, "%lf", &Cxs );
	ReadPar( File, "%lf", &Cys );
	ReadPar( File, "%lf", &Czs );
	ReadPar( File, "%lf", &Cls );
	ReadPar( File, "%lf", &Cms );
	ReadPar( File, "%lf", &Cns );
	ReadPar( File, "%lf", &Cxi );
	ReadPar( File, "%lf", &Czi );

	fclose( File );
	
	return 0;
}

int SDSA_FILE::Read( void )
{
	if( file_flag == 0 )return -1;
	return Read( file_name );
}

int SDSA_FILE::Write( const char* SDSA_file )
{
	FILE *File;
	File = fopen( SDSA_file, "w" );
	if( File == NULL )
		{
		fprintf( stderr, "[.sdsa] file open error to writing (%s)\n", SDSA_file );
		return (-1);
		}
		
	fprintf( File, "%14.8G # S\n", S );
	fprintf( File, "%14.8G # MAC\n", MAC );
	fprintf( File, "%14.8G # B\n", B );
	fprintf( File, "%14.8G # Swet\n", Swet );
	fprintf( File, "%14.8G %f # X,Y (ref)\n", X, Z );
	fprintf( File, "%14.8G # Alfa\n", Alfa );
	fprintf( File, "%14.8G # Beta\n", Beta );
	fprintf( File, "%14.8G # Mach\n", Mach );
	fprintf( File, "%14.8G # P\n", P );
	fprintf( File, "%14.8G # Q\n", Q );
	fprintf( File, "%14.8G # R\n", R );
	fprintf( File, "%14.8G # Cx\n", Cx );
	fprintf( File, "%14.8G # Cy\n", Cy );
	fprintf( File, "%14.8G # Cz\n", Cz );
	fprintf( File, "%14.8G # Cl\n", Cl );
	fprintf( File, "%14.8G # Cm\n", Cm );
	fprintf( File, "%14.8G # Cn\n", Cn );
	fprintf( File, "%14.8G # Cxs\n", Cxs );
	fprintf( File, "%14.8G # Cys\n", Cys );
	fprintf( File, "%14.8G # Czs\n", Czs );
	fprintf( File, "%14.8G # Cls\n", Cls );
	fprintf( File, "%14.8G # Cms\n", Cms );
	fprintf( File, "%14.8G # Cns\n", Cns );
	fprintf( File, "%14.8G # Cxi\n", Cxi );
	fprintf( File, "%14.8G # Czi\n", Czi );

	fclose( File );
	
	return 0;
}

int SDSA_FILE::Write( void )
{
	if( file_flag == 0 )return -1;
	return Write( file_name );
}

