/*********************************************************************/ 
/*                                                                   */ 
/*             Config files library - (C) TGG 2015                   */ 
/*                                                                   */ 
/*********************************************************************/ 
/*                          Warszawa, 2018                           */ 
/*********************************************************************/ 
/*                                                                   */
/* File: sdsa_file.h                                                 */
/*                                                                   */
/* Author: T.Grabowski                                               */
/*                                                                   */
/* Contents - sdsa file (simple output file) class declaration       */
/*                                                                   */ 
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/*                                                                   */

#ifndef _SDSA_FILE_H_
#define _SDSA_FILE_H_

#include "iofun.h"

/// Simple output file of Press program: reference values, aerodynamic coefficients, flow parameters

class SDSA_FILE : public IOFUN
{
	int  file_flag;         ///< flag indicating if the name of the file is defined (0/1 - NO/YES)
	char file_name[256];    ///< full pathname of the [.sdsa] file

	public:

	double S;         ///< reference area
	double MAC;       ///< mean aerodynamic chord
	double B;         ///< reference wingspan
	double Swet;      ///< wetted area
	double X;         ///< X coordinate fo the origin for moment components
	double Z;         ///< Z coordinate fo the origin for moment components
	double Alfa;      ///< angle of attack [deg]
	double Beta;      ///< sideslip angle [deg]
	double Mach;      ///< Mach number
	double P;         ///< roll rate [rad/s]
	double Q;         ///< pitch rate [rad/s]
	double R;         ///< yaw rate [rad/s]
	double Cx;        ///< drag coefficient (body axis system - pressure component)
	double Cy;        ///< sideforce coefficient (body axis system)
	double Cz;        ///< lift coefficient (body axis system)
	double Cl;        ///< rolling moment coefficient (body axis system)
	double Cm;        ///< pitching moment coefficient (body axis system)
	double Cn;        ///< yawing moment coefficient (body axis system)
	double Cxs;       ///< drag coefficient (stability axis system - pressure component)
	double Cys;       ///< sideforce coefficient (stability axis system)
	double Czs;       ///< lift coefficient (stability axis system)
	double Cls;       ///< rolling moment coefficient (stability axis system)
	double Cms;       ///< pitching moment coefficient (stability axis system)
	double Cns;       ///< yawing moment coefficient (stability axis system)
	double Cxi;       ///< induced drag coefficient
	double Czi;       ///< lift coefficients that relates to induced drag
	
	SDSA_FILE( void );
	~SDSA_FILE( void );

/// cleans local variables
	void Clean( void );
/// set pathname for the [.sdsa] file
	void SetName( const char *FileName );
/// reads the [.sdsa] file
	int Read( const char *FileName );
/// reads the [.sdsa] file defined by SetName function
	int Read( void );
/// writes the [.sdsa] file
	int Write( const char *FileName );
/// writes the [.sdsa] file defined by SetName function
	int Write( void );
};

#endif /*_SDSA_FILE_H_*/
