/*********************************************************************/ 
/*                                                                   */ 
/*             Config files library - (C) TGG 2018                   */ 
/*                                                                   */ 
/*********************************************************************/ 
/*                          Warsaw 2018                              */ 
/*********************************************************************/ 
/*                                                                   */
/* File: sdsa_params.cpp                                             */
/*                                                                   */
/* Written by: T.Grabowski                                           */
/*                                                                   */
/* Contents - class to read/write parameters file for batch mode     */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/*                                                                   */

#include "sdsa_params.h"

SDSA_PARAMS::SDSA_PARAMS( void ){ Clean(); }

SDSA_PARAMS::~SDSA_PARAMS( void ){}

void SDSA_PARAMS::Clean( void )
{
	dH0 = dH1 = dDH = 0.0;
	dV0 = dV1 = dDV = 0.0;
	dDA = 0.0;
	iSpeedType = 0;
	iAero = 0;
	
	iLQR = 0;
	iPilot_h = 0;
	iPilot_a = 0;
	iPilot_r = 0;
	iSAS_h = 0;
	iSAS_a = 0;
	iSAS_r = 0;
	iAct_h = 0;
	iAct_a = 0;
	iAct_r = 0;
	
	memset( OutFile, 0 , 256 );
}

int SDSA_PARAMS::Read( char* ParFile )
{
	FILE *File;
	File = fopen( ParFile, "r" );
	if( File == NULL )
		{
		fprintf( stderr, "SDSA params file open error (%s)\n", ParFile );
		return (-1);
		}

	ReadPar( File, "%lf %lf %lf", &dH0, &dH1, &dDH );
	ReadPar( File, "%lf %lf %lf", &dV0, &dV1, &dDV );
	ReadPar( File, "%lf", &dDA );
	ReadPar( File, "%d", &iSpeedType );
	ReadPar( File, "%d", &iAero );
	ReadPar( File, "%d", &iLQR );
	ReadPar( File, "%d %d %d", &iPilot_h, &iPilot_a, &iPilot_r );
	ReadPar( File, "%d %d %d", &iSAS_h, &iSAS_a, &iSAS_r );
	ReadPar( File, "%d %d %d", &iAct_h, &iAct_a, &iAct_r );
	ReadStr( File, OutFile );

	fclose( File );

	ClipFileName( OutFile );
	
	return (0);
}

int SDSA_PARAMS::Write( char *ParFile )
{
	FILE *File;
	File = fopen( ParFile, "w" );
	if( File == NULL )
		{
		fprintf( stderr, "SDSA params file open error to writing (%s)\n", ParFile );
		return (-1);
		}

	fprintf( File, "#       SDSA batch mode configuration file\n");
	fprintf( File, "%6.2f %6.2f %6.2f	# altitude range: mni, max, step\n", dH0, dH1, dDH );
	fprintf( File, "%6.2f %6.2f %6.2f	# airspeed range: mni, max, step\n", dV0, dV1, dDV );
	fprintf( File, "%6.2f	\t\t# aileron deflection angle\n", dDA );
	fprintf( File, "%d	\t\t# airspeed type: 0-TAS, 1-CAS\n", iSpeedType );
	fprintf( File, "%d	\t\t# aero data set (0,1)\n", iAero );
	fprintf( File, "%d	\t\t# LQR flag (0,1)\n", iLQR );
	fprintf( File, "%d %d %d	\t\t# Pilot in the loop (H,A,V)\n", iPilot_h, iPilot_a, iPilot_r );
	fprintf( File, "%d %d %d	\t\t# SAS in the loop (H,A,V)\n", iSAS_h, iSAS_a, iSAS_r );
	fprintf( File, "%d %d %d	\t\t# actuators in the loop (H,A,V)\n", iAct_h, iAct_a, iAct_r );
	fprintf( File, "%s	# output file\n", OutFile );

	fclose( File );
	
	return (0);
}
