/*********************************************************************/ 
/*                                                                   */ 
/*             Config files library - (C) TGG 2018                   */ 
/*                                                                   */ 
/*********************************************************************/ 
/*                          Warsaw 2018                              */ 
/*********************************************************************/ 
/*                                                                   */
/* File: sdsa_params.h                                               */
/*                                                                   */
/* Written by: T.Grabowski                                           */
/*                                                                   */
/* Contents - declaration of file with parameters for batch mode     */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/*                                                                   */

#ifndef  _SDSA_PARAMS_H_
#define  _SDSA_PARAMS_H_

#include "iofun.h"

/// \brief Parameters file for SDSA program to run SDSA in batch mode
///
/// Syntax of the batch mode: \b SDSA [title] [data pathname] [run_Type] [sdsa_params filename] [iGUI]
///   title - string: info about the analysed aircraft displayed in the status bar of the main SDSA window \n
///   data pathname - string: the name of folder that containing the data \n
///   runType - number: starting option: \n
///     0 - no selected, \n
///     1 - stability analysis, \n
///     2 - simulation, \n
///     3 - FCS definition, \n
///     4 - performance \n
///	  sdsa_params filename - name of the configuration file - this parameter forces exit from SDSA after stability computation \n
///	  iGUi - number: (option not active yet - GUI always present) \n
///	    0 - no GUI \n
///	    1 - GUI is displayed
///

class SDSA_PARAMS : public IOFUN
{
	public:

	double dH0;         ///< lower bound of hight of flight taken into the analysis
	double dH1;         ///< upper bound of hight of flight taken into the analysis
	double dDH;         ///< step of hight of flight taken into the analysis
	double dV0;         ///< lower bound of airspeed taken into the analysis
	double dV1;         ///< upper bound of airspeed taken into the analysis
	double dDV;         ///< step of airspeed taken into the analysis
	double dDA;         ///< aileron deflection [deg] (for roll response characteristics, must be grater than 0)
	int iSpeedType;     ///< airspeed type (0-TAS, 1-CAS)
	int iAero;          ///< number of aerodynamic data (1/2)
	                    
	int iLQR;           ///< LQR flag (0-1)
	int iPilot_h;       ///< pilot in the loop in elevator channel - flag (0-1)
	int iPilot_a;       ///< pilot in the loop in aileron channel - flag (0-1)
	int iPilot_r;       ///< pilot in the loop in rudder channel - flag (0-1)
	int iSAS_h;         ///< SAS in elevator channel - flag (0-1)
	int iSAS_a;         ///< SAS in aileron channel - flag (0-1) 
	int iSAS_r;         ///< SAS in rudder channel - flag (0-1)  
	int iAct_h;         ///< Actuator model in elevator channel - flag (0-1)
	int iAct_a;         ///< Actuator model in aileron channel - flag (0-1) 
	int iAct_r;         ///< Actuator model in rudder channel - flag (0-1)  
	
	char OutFile[256];  ///< pathname of output file

	SDSA_PARAMS( void );
	~SDSA_PARAMS( void );

/// cleans local variables
	void Clean();	
/// reads parameters file
	int Read( char* ParFile );
/// writes parameters file
	int Write( char* ParFile );
};

#endif

