/*********************************************************************/ 
/*                                                                   */ 
/*             Config files library - (C) TGG 2019                   */ 
/*                                                                   */ 
/*********************************************************************/ 
/*                          Warszawa, 2019                           */ 
/*********************************************************************/ 
/*                                                                   */
/* File: txt_out_file.cpp                                            */
/*                                                                   */
/* Author: T.Grabowski                                               */
/*                                                                   */
/* Contents - general output txt file                                */
/*                                                                   */ 
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/*                                                                   */

#include "txt_out_file.h"

TXT_OUT_FILE::TXT_OUT_FILE( int iTypeIn )
{
	iType = iTypeIn;
	Clean();
}

TXT_OUT_FILE::~TXT_OUT_FILE( void )
{
	CleanTabs();
}

void TXT_OUT_FILE::Clean( void )
{
	nRow = 0;
	nCol = NCOL_MAX;

	for( int i=0; i<nCol; i++ )sprintf(cName[i],"Var %d",i);
	
	file_flag = 0;
	dResult = NULL;
	iObj = NULL;
}

void TXT_OUT_FILE::CleanTabs( void )
{
	if( dResult )
		{
		for( int i=0; i<nCol; i++ )if( dResult[i] ){ delete [] dResult[i]; dResult[i] = NULL; }
		delete [] dResult; 
		dResult = NULL;
		}
	if( iObj )delete [] iObj;
	iObj = NULL;
}

void TXT_OUT_FILE::InitTabs( int n, int m )
{
	CleanTabs();

	nCol = n;
	nRow = m;
	dResult = new double*[nCol];
	for( int i=0; i<nCol; i++ )dResult[i] = new double[nRow];
	if ( iType ) iObj = new int[nRow];
}

void TXT_OUT_FILE::SetName( const char* TXT_file )
{
	strcpy( file_name, TXT_file );
	file_flag = 1;
}

char* TXT_OUT_FILE::GetName( void )
{
	return file_name;
}

int TXT_OUT_FILE::Read( const char* TXT_file )
{
	SetName( TXT_file);
	FILE *File;
	File = fopen( TXT_file, "r" );
	if( File == NULL )
		{
		fprintf( stderr, "File open error to reading (%s)\n", TXT_file );
		return (-1);
		}
		
	memset( cName, 0, NCOL_MAX*32 );
	CleanTabs();

	nCol = nColumns( File, cName ) - iType;
	nRow = nLines( File );
	
	dResult = new double*[nCol];
	for( int i=0; i<nCol; i++ )dResult[i] = new double[nRow];
	iObj = new int[nRow];
	
	for( int j=0; j<nRow; j++ )
		{
		if(iType)fscanf( File, "%d", &iObj[j] );
		for( int i=0; i<nCol; i++ )fscanf( File, "%lf", &dResult[i][j] );
		}

	fclose( File );
	
	return 0;
}

int TXT_OUT_FILE::Read( void )
{
	if( file_flag == 0 )return -1;
	return Read( file_name );
}

int TXT_OUT_FILE::Write( const char* TXT_file )
{
	SetName( TXT_file);
	FILE *File;
	File = fopen( TXT_file, "w" );
	if( File == NULL )
		{
		fprintf( stderr, "File open error to writing (%s)\n", TXT_file );
		return (-1);
		}

	for( int i=0; i<nCol+iType; i++ )fprintf( File, " %s", cName[i] );
	fprintf( File, "\n" );
	for( int j=0; j<nRow; j++ )
		{
		if(iType)fprintf( File, " %5d", iObj[j] );
		for( int i=0; i<nCol; i++ )fprintf( File, " %#16.8g", dResult[i][j] );
		fprintf( File, "\n" );
		}
	fclose( File );
	
	return 0;
}

int TXT_OUT_FILE::Write( void )
{
	if( file_flag == 0 )return -1;
	return Write( file_name );
}

