/*********************************************************************/ 
/*                                                                   */ 
/*             Config files library - (C) TGG 2019                   */ 
/*                                                                   */ 
/*********************************************************************/ 
/*                          Warszawa, 2019                           */ 
/*********************************************************************/ 
/*                                                                   */
/* File: txt_out_file.h                                              */
/*                                                                   */
/* Author: T.Grabowski                                               */
/*                                                                   */
/* Contents - general output txt file                                */
/*                                                                   */ 
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/*                                                                   */

#ifndef _TXT_OUT_FILE_H_
#define _TXT_OUT_FILE_H_

#include "iofun.h"

#define NCOL_MAX 32

/// General output text file

class TXT_OUT_FILE : public IOFUN
{
	int  file_flag;         ///< flag indicating if the name of the file is defined (0/1 - NO/YES)
	char file_name[256];    ///< full pathname of the output text file

	public:

	int nCol;               ///< number of columns/variables (up to NCOL_MAX)
	int nRow;               ///< number of rows
	
	char cName[NCOL_MAX][32];     ///< array of variable names - up to NCOL_MAX names, 32 characters each
	
	int iType;              ///< type of txt file - possible types: \n 
	                        ///< 0 - nCol x nRow - only double type values \n 
							///< 1 - (nCol-1) x nRow + first column int type
	int* iObj;              ///< pointer to array [nRow] first column value (int)
	double** dResult;       ///< pointer to array (size depends on iType) containing the results for all variables
	
/// constructor
	TXT_OUT_FILE( int iTypeIn = 0 );
	virtual ~TXT_OUT_FILE( void );

/// cleans local variables
	virtual void Clean( void );
/// cleans (deletes) arrays
	virtual void CleanTabs( void );
/// initialises arrays (n - number of columns, m - number of rows )
	virtual void InitTabs( int n, int m );
/// set pathname for of the output text file
	virtual void SetName( const char* FileName );
/// returns pathname of the output text file
	virtual char* GetName( void );
/// reads the [.czy] file
	virtual int Read( const char* FileName );
/// reads the [.czy] file defined by SetName function
	virtual int Read( void );
/// writes the [.czy] file
	virtual int Write( const char* FileName );
/// writes the [.czy] file defined by SetName function
	virtual int Write( void );
};

#endif /*_TXT_OUT_FILE_H_*/
